// Name: Jonathan Turner
// Class: CS 4306/01
// Term: Spring 2024
// Instructor: Dr. Haddad
// Assignment: 1
// IDE Name: IntelliJ
/*
Algorithm Design Block
    Algorithm title: String Anagram Checker


    Logical steps:
        Step 1: Gather user input for the two separate values to check and save these as Strings.
        Step 2: Remove Cases and Spaces from the Strings by using replaceAll and toLowerCase.
        Step 3: Convert the strings to two separate Char Arrays using toChar (A1 and A2).
        Step 4: Check the length of those Arrays, if they are different they are not anagrams and stop algorithm.
        Step 5: Create an outer (i) and inner (j) for loop that goes to the length of the array lengths (n).
        Step 6: Compare the A1[i] to A2[j], if value is the same mark A2[j] empty (' ') and increment comparisons.
        Step 7: After the for loops, make a boolean value to check if it is an anagram and set it to true.
        Step 8: Loop through A2, if there is a non-null value then they are not anagrams.


    Pseudocode Syntax:
        word1_string <- input
        word2_string <- input

        removeSpaces(word1_string)
        removeCases(word1_string)

        removeSpaces(word2_string)
        removeCases(word2_string)

        first_word <- word1_string.toChar
        second_word <- word2_string.toChar

        if first_word.length != second_word.length
            return False

        for i <- 1 to first_word
            for j <- 1 to second_word
                if first_word[i] = second_word[j] // Basic Operation
                    second_word[j] <- ' ';
                    break;
            end
        end

        result <- True
        for i <- 1 to second_word
            if second_word[i] != ' '
                result <- false
        end

        return result

*/
package Assignment1;

import java.util.Scanner;

public class AnagramsCheck {

    public static void main(String[] args) {
        AnagramsCheck program = new AnagramsCheck();
        program.start();
    }

    private String first;
    private String second;
    private int comparisons;

    public void start() {
        int option = getOption();
        while (option != 3) {
            executeOption(option);
            option = getOption();
        }
    }

    /**
     * Prints out the menu of options, asks for an input, and if that input is invalid it prints an error and
     * prompts the user again for the input.
     */
    private int getOption() {
        displayMenu(); // Prints out the menu

        // Asks for the option and begins the loop until a valid option is gathered.
        System.out.print("Enter option number: ");
        int input = getIntegerInput();
        while (input < 1 || input > 3) { // Compares it to the valid options available.
            System.out.println("\nPlease enter a valid input.");
            System.out.print("Enter option number: ");
            input = getIntegerInput();
        }
        System.out.println();
        return input;
    }

    /**
     * Gets user input to a number. A prompt must be provided prior to running this method.
     * If the input was not an integer/(unable to be autoboxed), returns -1.
     */
    private int getIntegerInput() {
        Scanner sc = new Scanner(System.in);
        try {
            String textInput = sc.nextLine();
            return Integer.parseInt(textInput);
        } catch (Exception e) {
            return -1;
        }
    }

    /**
     * Displays the menu options.
     */
    private void displayMenu() {
        System.out.println("-----------------MAIN MENU--------------");
        System.out.println("1. Read input string1 and string2");
        System.out.println("2. Run algorithm and display output");
        System.out.println("3. Exit program");
        System.out.println();
    }

    /**
     * Executs the option that is provided in the parameters.
     */
    private void executeOption(int option) {
        if (option == 1) { // Gets user input and sets the two strings.

            // Gets the first and second string as input.
            first = getString("first");
            System.out.println();
            second = getString("second");

        } else if (option == 2) { // Preforms the algorithm and displays the results.

            // Checks if the strings have been set prior to performing the algorithm
            if (first == null || first.isEmpty() ||
                    second == null || second.isEmpty()) {
                System.out.println("Please enter in the two strings prior to performing the algorithm.\n");
                return;
            }

            // Performs the algorithm and gathers results
            boolean result = performAlgorithm();
            String output = "Output:         ";
            if (result) {
                output += "Strings are anagrams";
            } else {
                output += "Strings are not anagrams";
            }

            // Prints results
            System.out.println("String 1:       " + first);
            System.out.println("String 2:       " + second);
            System.out.println(output);
            System.out.println("Comparisons:    " + comparisons);
        }
        System.out.println();
    }

    private String getString(String name) {
        System.out.println("Enter the " + name + " string.");
        System.out.print("String: ");
        Scanner sc = new Scanner(System.in);
        String input = sc.nextLine();

        while (input == null || input.isEmpty()) {
            System.out.println("Please enter a valid string.");
            System.out.print("String: ");
            input = sc.nextLine();
        }

        return input;
    }


    /**
     * Performs the algorithm in checking if two strings are anagrams.
     * It accomplishes this by first removing casing and spacing from the strings.
     *
     * Next it will go through two nested loops to see if each character from the first string
     * is inside the second string, if it is it sets the value to null.
     *
     * Lastly it loops through the second string to see if there are any non-null values,
     * if there is it returns false.
     */
    private boolean performAlgorithm() {

        // Removes the casing and spaces and sets each word in a char array.
        char[] first_word = first.toLowerCase().replaceAll(" ", "").toCharArray();
        char[] second_word = second.toLowerCase().replaceAll(" ", "").toCharArray();

        comparisons = 0;

        /*
        Checks if these two strings had the same number of non-whitespace characters.
        If not, they're not anagrams.
         */
        if (first_word.length != second_word.length) {
            return false;
        }

        // Loops through all the characters in the first (i) and second (j) word.
        for (int i = 0; i < first_word.length; i++) {
            for (int j = 0; j < second_word.length; j++) {
                comparisons++; // Counts the number of basic operations
                if (first_word[i] == second_word[j]) { // Basic Operation
                    second_word[j] = ' ';
                    break;
                }
            }
        }

        // Checks if there is any characters that were not set to ' ', if so not all characters matched.
        boolean valid = true;
        for (int i = 0; i < second_word.length; i++) {
            if (second_word[i] != ' ') {
                valid = false;
                break;
            }
        }
        return valid;
    }
}
